#include <msp/strings/format.h>
#include "binaryparser.h"
#include "dataerror.h"
#include "parser.h"
#include "statement.h"
#include "textparser.h"

using namespace std;

namespace Msp {
namespace DataFile {

Parser::Parser(IO::Base &i, const string &s):
	in(i),
	main_src(s),
	src(s),
	good(true),
	mode(new TextParser(in, src))
{ }

Parser::~Parser()
{
	delete mode;
}

Statement Parser::parse(bool raw)
{
	if(!good)
		throw logic_error("Parser::parse() !good");

	try
	{
		while(1)
		{
			Statement st = mode->parse(raw);
			if(st.keyword=="__bin")
			{
				delete mode;
				mode = new BinaryParser(in, src);
			}
			else if(st.keyword=="__text")
			{
				delete mode;
				mode = new TextParser(in, src);
			}
			else if(st.keyword=="__z")
				in.set_decompress();
			else if(st.keyword=="__src")
			{
				string s = st.args[0].get<string>();
				if(s.empty())
					src = main_src;
				else
					src = format("%s[%s]", main_src, s);
			}
			else if(st.keyword=="__end")
			{
				good = false;
				return raw ? st : Statement();
			}
			else
				return st;

			if(raw)
				return st;
		}
	}
	catch(const exception &e)
	{
		good = false;
		if(dynamic_cast<const data_error *>(&e))
			throw;
		else
			throw data_error(src, in.get_line_number(), e);
	}
}

} // namespace DataFile
} // namespace Msp
